/* Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * String controller
 */
let shareformKey = ""
let nodeIdParams = ""
let exeCount = true
let schemaListObj = {}
window.share && window.share.event.on("editCurrentNodeForm", (params)=> {
    shareformKey = params
})

window.share && window.share.event.on("editCurrentNodeId", (paramsId)=>{
    nodeIdParams = paramsId
})

angular.module('flowableModeler').controller('FlowableStringPropertyCtrl', [ '$scope', function ($scope) {

	$scope.shapeId = $scope.selectedShape.id;
	$scope.valueFlushed = false;
    /** Handler called when input field is blurred */
    $scope.inputBlurred = function() {
    	$scope.valueFlushed = true;
    	if ($scope.property.value) {
    		$scope.property.value = $scope.property.value.replace(/(<([^>]+)>)/ig,"");
    	}
        $scope.updatePropertyInModel($scope.property);
    };

    $scope.enterPressed = function(keyEvent) {
        // if enter is pressed
        if (keyEvent && keyEvent.which === 13) {
            keyEvent.preventDefault();
            $scope.inputBlurred(); // we want to do the same as if the user would blur the input field
        }
        // else; do nothing
    };
    
    $scope.$on('$destroy', function controllerDestroyed() {
    	if(!$scope.valueFlushed) {
    		if ($scope.property.value) {
        		$scope.property.value = $scope.property.value.replace(/(<([^>]+)>)/ig,"");
        	}
    		$scope.updatePropertyInModel($scope.property, $scope.shapeId);
    	}
    });

    // console.log($scope)

}]);

/*
 * Boolean controller
 */

angular.module('flowableModeler').controller('FlowableBooleanPropertyCtrl', ['$scope', function ($scope) {

    $scope.changeValue = function() {
        if ($scope.property.key === 'oryx-defaultflow' && $scope.property.value) {
            var selectedShape = $scope.selectedShape;
            if (selectedShape) {
                var incomingNodes = selectedShape.getIncomingShapes();
                if (incomingNodes && incomingNodes.length > 0) {
                    // get first node, since there can be only one for a sequence flow
                    var rootNode = incomingNodes[0];
                    var flows = rootNode.getOutgoingShapes();
                    if (flows && flows.length > 1) {
                        // in case there are more flows, check if another flow is already defined as default
                        for (var i = 0; i < flows.length; i++) {
                            if (flows[i].resourceId != selectedShape.resourceId) {
                                var defaultFlowProp = flows[i].properties.get('oryx-defaultflow');
                                if (defaultFlowProp) {
                                    flows[i].setProperty('oryx-defaultflow', false, true);
                                }
                            }
                        }
                    }
                }
            }
        }
        $scope.updatePropertyInModel($scope.property);
    };

}]);

/*
 * Text controller
 */

angular.module('flowableModeler').controller('FlowableTextPropertyCtrl', [ '$scope', '$modal', '$timeout', function($scope, $modal, $timeout) {

    var opts = {
        template:  'editor-app/configuration/properties/text-popup.html?version=' + Date.now(),
        scope: $scope,
        prefixEvent: 'textModalEvent'
    };
    
     $scope.$on('textModalEvent.hide.before', function() {
        $timeout(function() {
            $scope.property.mode = 'read';
        }, 0);
    });
    // Open the dialog
    _internalCreateModal(opts, $modal, $scope);
}]);

angular.module('flowableModeler').controller('FlowableTextPropertyPopupCtrl', ['$scope', function($scope) {

    $scope.save = function() {
        $scope.updatePropertyInModel($scope.property);
        $scope.close();
    };
    
    $scope.close = function() {
        $scope.property.mode = 'read';
        $scope.$hide();
    };
}]);

angular.module('flowableModeler').controller('FieldReadConfigPopupCtrl', ['$rootScope','$scope', '$http', function($rootScope, $scope, $http) {

    window.share && window.share.event.on("editCurrentNodeId", (params)=>{
        nodeIdParams = params
    })

    if (nodeIdParams && shareformKey) {
        $http.get(FLOWABLE.APP_URL.GetColumnConfig(nodeIdParams, shareformKey))
        .success((res)=>{
            if (res.FieldConfigList && res.FieldConfigList.length > 0) {
                res.FieldConfigList.forEach((item)=>{
                    $scope.property.properties.forEach((pItem)=>{
                        if (item.FieldCode === pItem.key) {
                            pItem.defaultUnit = item.Unit
                            pItem.defaultValue = item.Type
                            pItem.defaultChecked = item.IsShow === "1" ? true : false
                            pItem.defaultIsRequired = item.IsRequired === "true" ? true : false
                        }
                    })
                })
                schemaListObj = $scope.property

                $scope.childFieldObj = schemaListObj

            } else {
                $scope.property.properties.forEach(el => {
                    el.defaultValue = ""
                    el.defaultChecked = true
                    el.defaultUnit = ""
                    el.defaultIsRequired = false
                });
            }
        })
    }
    $scope.isShow = true

    $scope.toggleTable=function() {
        $scope.isShow = !$scope.isShow
    }

    $scope.changeValue = function(params) {
        const data = {
            ActID: $rootScope.currentReadResourceId,
            Config: "",
            FieldCode: params.key,
            FieldName: params.title,
            FieldType: params.type,
            FormKey: shareformKey,
            IsShow: params.defaultChecked ? 1: 0,
            Type: params.defaultValue,
            Unit: params.defaultUnit,
            IsRequired: params.defaultIsRequired
        }
        $http({method: 'POST', url: FLOWABLE.APP_URL.SaveFieldConfig(), data: data})
        .success(function(res) {
            // console.log(res)
        })
    }
}]);

angular.module('flowableModeler').controller('FieldWriteConfigPopupCtrl', ['$scope','$modal', '$timeout','$http', function($scope, $modal, $timeout, $http) {

    $scope.toggleTable=function() {
        $scope.isShow = !$scope.isShow
    }

    $scope.changeValue=function() {

    }
}]);

angular.module('flowableModeler').controller('FieldTextPopup', ['$rootScope','$scope', function($rootScope,$scope) {
    $scope.childFieldObj = $rootScope.curField
    // console.log($scope.childFieldObj)
    $scope.save = function() {
        $scope.updatePropertyInModel($scope.property);
        $scope.close();
    };
    
    $scope.close = function() {
        $scope.property.mode = 'read';
        $scope.$hide();
    };

    $scope.changeValue = function() {
        // console.log($scope)
    }
}]);


angular.module('flowableModeler').controller('ExtraFormReadConfig', ['$rootScope','$scope', '$http', function($rootScope,$scope, $http) {
    $scope.isShow = true
    $scope.componentList = [{
        name: '产品信息',
        component: 'ProductInfo',
        defaultChecked: false
    },
    {
        name: '客户信息',
        component: 'BusInfo',
        defaultChecked: false
    },
    {
        name: '商机信息',
        component: 'BusInfo',
        defaultChecked: false
    }]
    $scope.changeValue = function(params) {
        console.log(params)
    }
}]);